/*
 * tap.c - Kernel time handling tap
 *
 * Written 2002 by Werner Almesberger
 */


#define __KERNEL__
#include "linux/include/linux/time.h"
#include "linux/include/linux/timer.h"
#include "linux/include/linux/signal.h"
#include "linux/include/asm/param.h"

#include "timer.h"


#define __TRACE(s) \
  write(2,s,sizeof(s))
#define TRACE(s) __TRACE(s "\r\n")

#undef TRACE
#define TRACE(s)


const static int real_time = 0;
int use_sigalrm = 0;


extern int __real_setitimer(int which,const struct itimerval *value,
  struct itimerval *ovalue);

int __wrap_setitimer(int which,const struct itimerval *value,
  struct itimerval *ovalue)
{
    TRACE("setitimer");
    if (!real_time && (which == ITIMER_REAL || which == ITIMER_VIRTUAL)) {
	if (which == ITIMER_REAL) use_sigalrm = 1;
	return 0;
    }
    return __real_setitimer(which,value,ovalue);
}


extern void __real_add_timer(struct timer_list *timer);


void __wrap_add_timer(struct timer_list *timer)
{
    TRACE("add_timer");
//printk("add_timer(%p->%p)\n",timer,timer->function);
    if (!real_time) wake_kernel(timer->expires-jiffies);
    __real_add_timer(timer);
}


extern int __real_mod_timer(struct timer_list *timer, unsigned long expires);

int __wrap_mod_timer(struct timer_list *timer,unsigned long expires)
{
    TRACE("mod_timer");
//printk("mod_timer(%p->%p)\n",timer,timer->function);
    if (!real_time) wake_kernel(expires-jiffies);
    return __real_mod_timer(timer,expires);
}


extern void __real_idle_sleep(int nap);

void __wrap_idle_sleep(int nap)
{
    TRACE("idle_sleep");
    if (real_time) __real_idle_sleep(nap);
    else wait_kernel();
}


extern void __real_do_gettimeofday(struct timeval *tv);

void __wrap_do_gettimeofday(struct timeval *tv)
{
    TRACE("do_gettimeofday");
    if (real_time) __real_do_gettimeofday(tv);
    else *tv = now;
}
